#!/usr/bin/env python3
#
#

import sys, os, getopt, struct, time
import json
import zmq

def usage(tool):
    print("")
    print(f"Usage: {tool} [options] [query]")
    print("")

    print("IP Lookup query options:")
    print("  -a, --prefix".ljust(25) + "IP address/prefix to look up")
    print("  -d, --rd".ljust(25) + "Route Distinguisher to look up")
    print("  -r, --peer".ljust(25) + "BGP peer to look up")
    print("  -R, --peer-port".ljust(25) + "TCP port of the BGP peer (for BGP through NAT/proxy scenarios)")
    print("")
    print("Get Peers query options:")
    print("  -g, --get-peers".ljust(25) + "Get the list of BGP peers at the Looking Glass")
    print("")
    print("General options:")
    print("  -z, --zmq-host".ljust(25) + "Looking Glass IP address [default: 127.0.0.1]")
    print("  -Z, --zmq-port".ljust(25) + "Looking Glass port [default: 17900]")
    print("  -u, --zmq-user".ljust(25) + "Looking Glass username [default: none]")
    print("  -p, --zmq-passwd".ljust(25) + "Looking Glass password [default: none]")
    print("")
    print("  -h, --help".ljust(25) + "Print this help")

def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "ha:d:r:R:z:Z:u:p:g", ["help", "prefix=", "rd=",
                "peer=", "peer-port=", "zmq-host=", "zmq-port=", "zmq-user=", "zmq-passwd=",
                "get-peers="])
    except getopt.GetoptError as err:
        print(str(err))
        usage(sys.argv[0])
        sys.exit(2)

    # Vars for connecting to the LG
    lgHost = "127.0.0.1"
    lgPort = "17900"
    lgUser = b"" 
    lgPasswd = b"" 
    queryTypeObj = {}
    queryDataObj = {}
    queryTypeJsonObj = ""
    queryDataJsonObj = ""

    # Vars for IP Lookup query
    queryIpl = 0
    queryIplPrefix = "" 
    queryIplRd = "" 
    queryIplPeer = "" 
    queryIplPeerPort = ""
    queryIplRequiredCl = 0

    # Vars for Get Peers query
    queryGp = 0
    
    for o, a in opts:
        if o in ("-h", "--help"):
            usage(sys.argv[0])
            sys.exit()
        elif o in ("-a", "--prefix"):
            queryIplRequiredCl += 1
            queryIpl = 1
            queryIplPrefix = a
        elif o in ("-d", "--rd"):
            queryIpl = 1
            queryIplRd = a
        elif o in ("-r", "--peer"):
            queryIplRequiredCl += 1
            queryIpl = 1
            queryIplPeer = a
        elif o in ("-R", "--peer-port"):
            queryIpl = 1
            queryIplPeerPort = a
        elif o in ("-g", "--get-peers"):
            queryGp = 1
        elif o in ("-z", "--zmq-host"):
            lgHost = a
        elif o in ("-Z", "--zmq-port"):
            lgPort = a
        elif o in ("-u", "--zmq-user"):
            lgUser = a.encode()
        elif o in ("-p", "--zmq-passwd"):
            lgPasswd = a.encode()
        else:
            assert False, "unhandled option"

    # Validations
    if not queryGp and not queryIpl:
        print("ERROR: no query specificed.")
        usage(sys.argv[0])
        sys.exit(1)

    if queryGp and queryIpl:
        print("ERROR: IP Lookup and Get Peers queries are mutual exclusive. Please select only one.")
        sys.exit(1)

    if queryIpl and queryIplRequiredCl < 2: 
        print("ERROR: Missing required arguments (-a, -r) for IP Lookup query")
        usage(sys.argv[0])
        sys.exit(1)

    # Craft query
    if queryIpl:
        queryTypeObj['query_type'] = 1
        queryTypeObj['queries'] = 1
        queryDataObj['peer_ip_src'] = queryIplPeer
        queryDataObj['ip_prefix'] = queryIplPrefix

        if len(queryIplRd):
            queryDataObj['rd'] = queryIplRd

        if len(queryIplPeerPort):
            queryDataObj['peer_tcp_port'] = queryIplPeerPort

        queryTypeJsonObj = json.dumps(queryTypeObj).encode()
        queryDataJsonObj = json.dumps(queryDataObj).encode()


    if queryGp:
        queryTypeObj['query_type'] = 2
        queryTypeObj['queries'] = 1
        queryTypeJsonObj = json.dumps(queryTypeObj).encode()


    # Connect to LG, send request, read reply 
    lgClientZmqCtx = zmq.Context()
    lgClientZmqReq = lgClientZmqCtx.socket(zmq.REQ)

    if len(lgUser):
        lgClientZmqReq.plain_username = lgUser

    if len(lgPasswd):
        lgClientZmqReq.plain_password = lgPasswd

    lgConnectStr = "tcp://" + lgHost + ":" + lgPort
    lgClientZmqReq.connect(lgConnectStr)

    if len(queryDataJsonObj):
        lgClientZmqReq.send(queryTypeJsonObj, zmq.SNDMORE)
        lgClientZmqReq.send(queryDataJsonObj)
    else:
        lgClientZmqReq.send(queryTypeJsonObj)

    repResults = lgClientZmqReq.recv();
    if len(repResults):
        repType = 0
        repNum = 0

        repResultsJsonObj = json.loads(repResults)
        if not 'query_type' in repResultsJsonObj.keys():
            print("WARN: no 'query_type' element.")
            sys.exit(1)
        else:
            repType = repResultsJsonObj['query_type']

        if not 'results' in repResultsJsonObj.keys():
            print("WARN: no 'results' element.")
            sys.exit(1)
        else:
            repNum = repResultsJsonObj['results']

        print(repResults)

        repIdx = 0
        while repIdx < repNum:
            repData = lgClientZmqReq.recv();
            print(repData)

            repIdx += 1

if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# Pika is a pure-Python implementation of the AMQP 0-9-1 protocol and
# is available at:
# https://pypi.python.org/pypi/pika
# http://www.rabbitmq.com/tutorials/tutorial-one-python.html
#
# UltraJSON, an ultra fast JSON encoder and decoder, is available at:
# https://pypi.python.org/pypi/ujson
#
# The Apache Avro Python module is available at: 
# https://avro.apache.org/docs/1.8.1/gettingstartedpython.html
#
# Binding to the routing key specified by amqp_routing_key (by default 'acct')
# allows to receive messages published by an 'amqp' plugin, in JSON format.
# Similarly for BGP daemon bgp_*_routing_key and BMP daemon bmp_*_routing_key.
#
# Binding to the reserved exchange 'amq.rabbitmq.trace' and to routing keys
# 'publish.pmacct' or 'deliver.<queue name>' allows to receive a copy of the
# messages that published via a specific exchange or delivered to a specific
# queue. RabbitMQ Firehose Tracer feature should be enabled first with the
# following command:
#
# 'rabbitmqctl trace_on' enables RabbitMQ Firehose tracer
# 'rabbitmqctl list_queues' lists declared queues
#
# Two pipelines are supported in this script:
# * RabbitMQ -> REST API
# * RabbitMQ -> stdout
#
# Two data encoding formats are supported in this script:
# * JSON
# * Apache Avro

import sys, os, getopt, pika, StringIO, time
import ujson as json

try:
	import avro.io
	import avro.schema
	import avro.datafile
	avro_available = True
except ImportError:
	avro_available = False

avro_schema = None
http_url_post = None
print_stdout = 0
print_stdout_num = 0
print_stdout_max = 0
convert_to_json_array = 0
stats_interval = 0
time_count = 0
elem_count = 0

def usage(tool):
	print ""
	print "Usage: %s [Args]" % tool
	print ""

	print "Mandatory Args:"
	print "  -e, --exchange".ljust(25) + "Define the exchange to bind to"
	print "  -k, --routing_key".ljust(25) + "Define the routing key to use"
	print "  -q, --queue".ljust(25) + "Specify the queue to declare"
	print ""
	print "Optional Args:"
	print "  -h, --help".ljust(25) + "Print this help"
	print "  -H, --host".ljust(25) + "Define RabbitMQ broker host [default: 'localhost']"
	print "  -p, --print".ljust(25) + "Print data to stdout"
	print "  -n, --num".ljust(25) + "Number of rows to print to stdout [default: 0, ie. forever]"
	print "  -u, --url".ljust(25) + "Define a URL to HTTP POST data to" 
	print "  -a, --to-json-array".ljust(25) + "Convert list of newline-separated JSON objects in a JSON array"
	print "  -s, --stats-interval".ljust(25) + "Define a time interval, in secs, to get statistics to stdout"
	print "  -P, --pidfile".ljust(25) + "Set a pidfile to record active processes PID"
	if avro_available:
		print "  -d, --decode-with-avro".ljust(25) + "Define the file with the " \
		      "schema to use for decoding Avro messages"

def post_to_url(http_req, value):
	try:
		urllib2.urlopen(http_req, value)
	except urllib2.HTTPError, err:
		print "WARN: urlopen() returned HTTP error code:", err.code
		sys.stdout.flush()
	except urllib2.URLError, err:
		print "WARN: urlopen() returned URL error reason:", err.reason
		sys.stdout.flush()

def callback(ch, method, properties, body):
        global avro_schema
        global http_url_post
        global print_stdout
        global print_stdout_num
        global print_stdout_max
        global convert_to_json_array
        global stats_interval
        global time_count
        global elem_count

	if stats_interval:
		time_now = int(time.time())

	if avro_schema:
		inputio = StringIO.StringIO(body)
		decoder = avro.io.BinaryDecoder(inputio)
		datum_reader = avro.io.DatumReader(avro_schema)

		avro_data = []
		while inputio.tell() < len(inputio.getvalue()):
			x = datum_reader.read(decoder)
			avro_data.append(str(x))

		#
		# XXX: data enrichments, manipulations, correlations, filtering etc. go here
		#

		if stats_interval:
			elem_count += len(avro_data)

		if print_stdout:
			print " [x] Received %r" % (",".join(avro_data),)
			sys.stdout.flush()
			print_stdout_num += 1
			if (print_stdout_max == print_stdout_num):
				sys.exit(0)

		if http_url_post:
			http_req = urllib2.Request(http_url_post)
			http_req.add_header('Content-Type', 'application/json')
			post_to_url(http_req, ("\n".join(avro_data)))
	else:
		value = body

		try:
			jsonObj = json.loads(value)
		except ValueError:
			print("ERROR: json.loads: '%s'. Skipping." % value)
			return

		#
		# XXX: data enrichments, manipulations, correlations, filtering etc. go here
		#

		if stats_interval:
			elem_count += value.count('\n')
			elem_count += 1

		if convert_to_json_array:
			value = "[" + value + "]"
			value = value.replace('\n', ',\n')
			value = value.replace(',\n]', ']')

		if print_stdout:
			print " [x] Received %r" % (value,)
			sys.stdout.flush()
			print_stdout_num += 1
			if (print_stdout_max == print_stdout_num):
				sys.exit(0)

		if http_url_post:
			http_req = urllib2.Request(http_url_post)
			http_req.add_header('Content-Type', 'application/json')
			post_to_url(http_req, value)

	if stats_interval:
		if time_now >= (time_count + stats_interval):
			print("INFO: stats: [ interval=%d records=%d ]" % (stats_interval, elem_count))
			sys.stdout.flush()
			time_count = time_now
			elem_count = 0

def main():
        global avro_schema
        global http_url_post
        global print_stdout
        global print_stdout_num
        global print_stdout_max
        global convert_to_json_array
        global stats_interval
        global time_count
        global elem_count

	try:
		opts, args = getopt.getopt(sys.argv[1:], "he:k:q:H:u:d:pn:as:P:", ["help",
				"exchange=", "routing_key=", "queue=", "host=", "url=",
				"decode-with-avro=", "print=", "num=", "to-json-array=",
				"stats-interval=", "pidfile="])
	except getopt.GetoptError as err:
		# print help information and exit:
		print str(err) # will print something like "option -a not recognized"
		usage(sys.argv[0])
		sys.exit(2)

	amqp_exchange = None
	amqp_routing_key = None
	amqp_queue = None
	amqp_host = "localhost"
	pidfile = None
 	
	required_cl = 0

	for o, a in opts:
		if o in ("-h", "--help"):
			usage(sys.argv[0])
			sys.exit()
		elif o in ("-e", "--exchange"):
			required_cl += 1
            		amqp_exchange = a
		elif o in ("-k", "--routing_key"):
			required_cl += 1
            		amqp_routing_key = a
		elif o in ("-q", "--queue"):
			required_cl += 1
            		amqp_queue = a
		elif o in ("-H", "--host"):
            		amqp_host = a
		elif o in ("-u", "--url"):
			http_url_post = a
		elif o in ("-p", "--print"):
			print_stdout = 1
		elif o in ("-n", "--num"):
			print_stdout_max = int(a)
		elif o in ("-a", "--to-json-array"):
			convert_to_json_array = 1
		elif o in ("-s", "--stats-interval"):
			stats_interval = int(a)
			if stats_interval < 0:
				sys.stderr.write("ERROR: `--stats-interval` must be positive\n")
				sys.exit(1)
		elif o in ("-d", "--decode-with-avro"):
			if not avro_available:
				sys.stderr.write("ERROR: `--decode-with-avro` given but Avro package was "
						"not found\n")
				sys.exit(1)

                        if not os.path.isfile(a):
				sys.stderr.write("ERROR: '%s' does not exist or is not a file\n" % (a,))
				sys.exit(1)

			with open(a) as f:
				avro_schema = avro.schema.parse(f.read())
		elif o in ("-P", "--pidfile"):
			pidfile = a
	        else:
			assert False, "unhandled option"

	amqp_type = "direct"
 	
	if (required_cl < 3): 
		print "ERROR: Missing required arguments"
		usage(sys.argv[0])
		sys.exit(1)

	if pidfile:
		pidfile_f = open(pidfile, 'w')
		pidfile_f.write(str(mypid))
		pidfile_f.write("\n")
		pidfile_f.close()

	connection = pika.BlockingConnection(pika.ConnectionParameters(host=amqp_host))
	channel = connection.channel()

	channel.exchange_declare(exchange=amqp_exchange, type=amqp_type)

	channel.queue_declare(queue=amqp_queue)

	channel.queue_bind(exchange=amqp_exchange, routing_key=amqp_routing_key, queue=amqp_queue)

	if print_stdout:
		print ' [*] Example inspired from: http://www.rabbitmq.com/getstarted.html'
		print ' [*] Waiting for messages on E =', amqp_exchange, ',', amqp_type, 'RK =', amqp_routing_key, 'Q =', amqp_queue, 'H =', amqp_host, '. Edit code to change any parameter. To exit press CTRL+C'
		sys.stdout.flush()

	if stats_interval:
		elem_count = 0
		time_count = int(time.time())

	channel.basic_consume(callback, queue=amqp_queue, no_ack=True)

	channel.start_consuming()

if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# Confluent Kafka Python module is available at:
# https://github.com/confluentinc/confluent-kafka-python
#
# UltraJSON, an ultra fast JSON encoder and decoder, is available at:
# https://pypi.python.org/pypi/ujson
#
# Binding to the topic specified by kafka_topic (by default 'acct') allows to
# receive messages published by a 'kafka' plugin, in JSON format. Similarly for
# BGP daemon bgp_*_topic and BMP daemon bmp_*_topic.
#
# Three pipelines are supported in this script:
# * Kafka -> Kafka 
# * Kafka -> REST API
# * Kafka -> stdout
#
# A single data encoding format is supported in this script:
# * JSON

import sys, os, getopt, StringIO, time, urllib2
import confluent_kafka
import ujson as json
import uuid

def usage(tool):
	print ""
	print "Usage: %s [Args]" % tool
	print ""

	print "Mandatory Args:"
	print "  -t, --topic".ljust(25) + "Define the topic to consume from"
	print ""
	print "Optional Args:"
	print "  -h, --help".ljust(25) + "Print this help"
	print "  -g, --group_id".ljust(25) + "Specify the consumer Group ID"
	print "  -e, --earliest".ljust(25) + "Set consume topic offset to 'earliest' [default: 'latest']"
	print "  -H, --host".ljust(25) + "Define Kafka broker host [default: '127.0.0.1:9092']"
	print "  -p, --print".ljust(25) + "Print data to stdout"
	print "  -n, --num".ljust(25) + "Number of rows to print to stdout [default: 0, ie. forever]"
	print "  -T, --produce-topic".ljust(25) + "Define a topic to produce to"
	print "  -u, --url".ljust(25) + "Define a URL to HTTP POST data to"
	print "  -a, --to-json-array".ljust(25) + "Convert list of newline-separated JSON objects in a JSON array"
	print "  -s, --stats-interval".ljust(25) + "Define a time interval, in secs, to get statistics to stdout"
	print "  -P, --pidfile".ljust(25) + "Set a pidfile to record active processes PID"


def post_to_url(http_req, value):
	try:
		urllib2.urlopen(http_req, value)
	except urllib2.HTTPError, err:
		print "WARN: urlopen() returned HTTP error code:", err.code
		sys.stdout.flush()
	except urllib2.URLError, err:
		print "WARN: urlopen() returned URL error reason:", err.reason
		sys.stdout.flush()

def main():
	try:
		opts, args = getopt.getopt(sys.argv[1:], "ht:T:pin:g:H:d:eu:as:r:P:", ["help", "topic=",
				"group_id=", "host=", "earliest=", "url=", "produce-topic=", "print=",
				"num=", "to-json-array=", "stats-interval=", "pidfile="])
	except getopt.GetoptError as err:
		# print help information and exit:
		print str(err) # will print something like "option -a not recognized"
		usage(sys.argv[0])
		sys.exit(2)

	mypid = os.getpid()
	kafka_topic = None
	kafka_group_id = uuid.uuid1() 
	kafka_host = "127.0.0.1:9092"
	kafka_produce_topic = None
	topic_offset = "latest"
	http_url_post = None
	print_stdout = 0
        print_stdout_num = 0
        print_stdout_max = 0
	convert_to_json_array = 0
	stats_interval = 0
	pidfile = None
 	
	required_cl = 0

	for o, a in opts:
		if o in ("-h", "--help"):
			usage(sys.argv[0])
			sys.exit()
		elif o in ("-t", "--topic"):
			required_cl += 1
            		kafka_topic = a
		elif o in ("-T", "--produce-topic"):
            		kafka_produce_topic = a
		elif o in ("-p", "--print"):
            		print_stdout = 1
		elif o in ("-n", "--num"):
			print_stdout_max = int(a)
		elif o in ("-g", "--group_id"):
            		kafka_group_id = a
		elif o in ("-H", "--host"):
            		kafka_host = a
		elif o in ("-e", "--earliest"):
			topic_offset = "earliest"
		elif o in ("-u", "--url"):
			http_url_post = a
		elif o in ("-a", "--to-json-array"):
			convert_to_json_array = 1
		elif o in ("-s", "--stats-interval"):
			stats_interval = int(a)
			if stats_interval < 0:
				sys.stderr.write("ERROR: `-s`, `--stats-interval` must be positive\n")
				sys.exit(1)
		elif o in ("-P", "--pidfile"):
			pidfile = a
		else:
			assert False, "unhandled option"

	if required_cl < 1: 
		print "ERROR: Missing required arguments"
		usage(sys.argv[0])
		sys.exit(1)

	if pidfile:
		pidfile_f = open(pidfile, 'w')
		pidfile_f.write(str(mypid))
		pidfile_f.write("\n")
		pidfile_f.close()

	consumer_conf = { 'bootstrap.servers': kafka_host,
			  'group.id': kafka_group_id,
			  'default.topic.config': {
				'auto.offset.reset': topic_offset
			  }
			}

	consumer = confluent_kafka.Consumer(**consumer_conf)
	consumer.subscribe([kafka_topic])

	producer_conf = { 'bootstrap.servers': kafka_host }
	if kafka_produce_topic:
		producer = confluent_kafka.Producer(**producer_conf)

	if stats_interval:
		elem_count = 0
		time_count = int(time.time())

	while True:
		message = consumer.poll()
		value = message.value().decode('utf-8')

		if stats_interval:
			time_now = int(time.time())

		if len(value):
			try:
				jsonObj = json.loads(value)
			except ValueError:
				print("ERROR: json.loads: '%s'. Skipping." % value)
				continue

			if 'event_type' in jsonObj:
				if jsonObj['event_type'] == "purge_init":
					continue
				elif jsonObj['event_type'] == "purge_close":
					continue
				elif jsonObj['event_type'] == "purge":
					pass
				else:
					print("WARN: json.loads: flow record with unexpected event_type '%s'. Skipping." % jsonObj['event_type'])
					continue
			else:
				print("WARN: json.loads: flow record with no event_type field. Skipping.")
				continue

			#
			# XXX: data enrichments, manipulations, correlations, filtering, etc. go here
			#

			if stats_interval:
				elem_count += 1

			if convert_to_json_array:
				value = "[" + value + "]"
				value = value.replace('\n', ',\n')
				value = value.replace(',\n]', ']')

			if print_stdout:
				print("%s:%d:%d: pid=%d key=%s value=%s" % (message.topic(), message.partition(),
						message.offset(), mypid, str(message.key()), value))
				sys.stdout.flush()
				print_stdout_num += 1
				if (print_stdout_max == print_stdout_num):
					sys.exit(0)

			if http_url_post:
				http_req = urllib2.Request(http_url_post)
				http_req.add_header('Content-Type', 'application/json')
				post_to_url(http_req, value)

			if kafka_produce_topic:
				producer.produce(kafka_produce_topic, value)
				producer.poll(0)

		if stats_interval:
			if time_now >= (time_count + stats_interval):
				print("INFO: stats: [ time=%s interval=%d records=%d pid=%d ]" %
					(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time_now)), stats_interval, elem_count), mypid)
				sys.stdout.flush()
				time_count = time_now
				elem_count = 0

if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# If missing 'avro' read how to download it at: 
# https://avro.apache.org/docs/1.8.1/gettingstartedpython.html
#
# 'avro' package is deemed slow. For production scenarios, do consider
# implementing this script using the 'fastavro' Python package that is
# documented here: https://fastavro.readthedocs.io/en/latest/

import sys, os, getopt, io
from avro.datafile import DataFileReader
from avro.io import DatumReader
import avro.schema

def usage(tool):
	print ""
	print "Usage: %s [Args]" % tool
	print ""

	print "Mandatory Args:"
	print "  -i, --input-file".ljust(25) + "Input file in Avro format"
	print "  -s, --schema".ljust(25) + "Schema to decode input file (if not included)"
	print ""
	print "Optional Args:"
	print "  -h, --help".ljust(25) + "Print this help"

def main():
	try:
		opts, args = getopt.getopt(sys.argv[1:], "hi:s:", ["help", "input-file=",
						"schema="])
	except getopt.GetoptError as err:
		# print help information and exit:
		print str(err) # will print something like "option -a not recognized"
		usage(sys.argv[0])
		sys.exit(2)

	avro_file = None
	avro_schema_file = None

	required_cl = 0

	for o, a in opts:
		if o in ("-h", "--help"):
			usage(sys.argv[0])
			sys.exit()
		elif o in ("-i", "--input-file"):
			required_cl += 1
            		avro_file = a
		elif o in ("-s", "--schema"):
			avro_schema_file = a
		else:
			assert False, "unhandled option"

	if (required_cl < 1): 
		print "ERROR: Missing required argument"
		usage(sys.argv[0])
		sys.exit(1)

	if not avro_schema_file:
		reader = DataFileReader(open(avro_file, "r"), DatumReader())
		for datum in reader:
			print datum
		reader.close()
	else:
		reader_schema = open(avro_schema_file, "r")
		avro_schema = reader_schema.read()
		reader_schema.close()
		parsed_avro_schema = avro.schema.parse(avro_schema)

		with open(avro_file, "rb") as reader_data:
			inputio = io.BytesIO(reader_data.read())
			decoder = avro.io.BinaryDecoder(inputio)
			reader = avro.io.DatumReader(parsed_avro_schema)
			while inputio.tell() < len(inputio.getvalue()):
				avro_datum = reader.read(decoder)
				print avro_datum
		reader_data.close()

if __name__ == "__main__":
    main()
